@@ -1,14 +1,14 @@
 // modules
-mod parser;
+mod model;
 mod query;
-mod rcdom;
+
+use std::collections::HashMap;
 
 // re-export
-pub use parser::{Article, CollapseState, HnText, Story};
-pub use query::StoryNumericFilters;
+pub use query::{StoryNumericFilters, StorySortMode};
 
 use crate::prelude::*;
-use parser::*;
+use model::*;
 use rayon::prelude::*;
 
 const HN_ALGOLIA_PREFIX: &str = "https://hn.algolia.com/api/v1";
@@ -21,9 +21,6 @@ pub const SEARCH_LIMIT: usize = 15;
 
 static CLIENT: once_cell::sync::OnceCell<HNClient> = once_cell::sync::OnceCell::new();
 
-pub type CommentSender = crossbeam_channel::Sender<Vec<HnText>>;
-pub type CommentReceiver = crossbeam_channel::Receiver<Vec<HnText>>;
-
 /// HNClient is a HTTP client to communicate with Hacker News APIs.
 #[derive(Clone)]
 pub struct HNClient {
@@ -59,29 +56,51 @@ impl HNClient {
     where
         T: serde::de::DeserializeOwned,
     {
-        let request_url = format!("{}/items/{}", HN_ALGOLIA_PREFIX, id);
+        let request_url = format!("{HN_ALGOLIA_PREFIX}/items/{id}");
         let item = log!(
             self.client.get(&request_url).call()?.into_json::<T>()?,
-            format!("get HN item (id={}) using {}", id, request_url)
+            format!("get HN item (id={id}) using {request_url}")
         );
         Ok(item)
     }
 
-    /// Lazily load a story's comments
+    pub fn get_story_hidden_data(&self, story_id: u32) -> Result<StoryHiddenData> {
+        // Parallelize two tasks using [`rayon::join`](https://docs.rs/rayon/latest/rayon/fn.join.html)
+        let (content, comment_receiver) = rayon::join(
+            || {
+                log!(
+                    self.get_story_page_content(story_id),
+                    format!("get story (id={story_id}) page content")
+                )
+            },
+            || self.lazy_load_story_comments(story_id),
+        );
+        let content = content?;
+        let comment_receiver = comment_receiver?;
+
+        let vote_state = self.parse_story_vote_data(&content)?;
+
+        Ok(StoryHiddenData {
+            comment_receiver,
+            vote_state,
+        })
+    }
+
     pub fn lazy_load_story_comments(&self, story_id: u32) -> Result<CommentReceiver> {
-        let request_url = format!("{}/item/{}.json", HN_OFFICIAL_PREFIX, story_id);
+        // retrieve the top comments of a story
+        let request_url = format!("{HN_OFFICIAL_PREFIX}/item/{story_id}.json");
         let mut ids = log!(
             self.client
                 .get(&request_url)
                 .call()?
                 .into_json::<HNStoryResponse>()?
                 .kids,
-            format!("get story (id={}) using {}", story_id, request_url)
+            format!("get story (id={story_id}) using {request_url}")
         );
 
         let (sender, receiver) = crossbeam_channel::bounded(32);
 
-        // loads first 5 comments to ensure the corresponding `CommentView` has data to render
+        // loads the first 5 top comments to ensure the corresponding `CommentView` has data to render
         self.load_comments(&sender, &mut ids, 5)?;
         std::thread::spawn({
             let client = self.clone();
@@ -129,13 +148,13 @@ impl HNClient {
 
     /// Get a story based on its id
     pub fn get_story_from_story_id(&self, id: u32) -> Result<Story> {
-        let request_url = format!("{}/search?tags=story,story_{}", HN_ALGOLIA_PREFIX, id);
+        let request_url = format!("{HN_ALGOLIA_PREFIX}/search?tags=story,story_{id}");
         let response = log!(
             self.client
                 .get(&request_url)
                 .call()?
                 .into_json::<StoriesResponse>()?,
-            format!("get story (id={}) using {}", id, request_url)
+            format!("get story (id={id}) using {request_url}")
         );
 
         match <Vec<Story>>::from(response).pop() {
@@ -166,19 +185,19 @@ impl HNClient {
                 .call()?
                 .into_json::<StoriesResponse>()?,
             format!(
-                "get matched stories with query {} (by_date={}, page={}) using {}",
-                query, by_date, page, request_url
+                "get matched stories with query {query} (by_date={by_date}, page={page}) using {request_url}"
             )
         );
 
         Ok(response.into())
     }
 
-    // reorder the front_page stories to follow the same order
-    // as in the official Hacker News site.
-    // Needs to do this because stories returned by Algolia APIs
-    // are sorted by `points`.
-    fn reoder_front_page_stories(&self, stories: Vec<Story>, ids: &[u32]) -> Vec<Story> {
+    /// Reorder a list of stories to follow the same order as another list of story IDs.
+    ///
+    /// Needs to do this because stories returned by Algolia APIs are sorted by `points`,
+    /// reoder those stories to match the list shown up in the HackerNews website,
+    /// which has the same order as the list of IDs returned from the official API.
+    fn reorder_stories_based_on_ids(&self, stories: Vec<Story>, ids: &[u32]) -> Vec<Story> {
         let mut stories = stories;
         stories.sort_by(|story_x, story_y| {
             let story_x_pos = ids
@@ -199,20 +218,30 @@ impl HNClient {
         stories
     }
 
-    // retrieve a list of front_page story ids using HN Official API then
-    // compose a HN Algolia API to retrieve the corresponding stories.
-    fn get_front_page_stories(
+    /// Retrieve a list of story IDs given a story tag using the HN Official API
+    /// then compose a HN Algolia API to retrieve the corresponding stories' data.
+    fn get_stories_no_sort(
         &self,
+        tag: &str,
         page: usize,
         numeric_filters: query::StoryNumericFilters,
     ) -> Result<Vec<Story>> {
-        let request_url = format!("{}/topstories.json", HN_OFFICIAL_PREFIX);
+        // get the HN official API's endpoint based on query's story tag
+        let endpoint = match tag {
+            "front_page" => "/topstories.json",
+            "ask_hn" => "/askstories.json",
+            "show_hn" => "/showstories.json",
+            _ => {
+                anyhow::bail!("unsupported story tag {tag}");
+            }
+        };
+        let request_url = format!("{HN_OFFICIAL_PREFIX}{endpoint}");
         let stories = log!(
             self.client
                 .get(&request_url)
                 .call()?
                 .into_json::<Vec<u32>>()?,
-            format!("get front page stories using {}", request_url)
+            format!("get {tag} story IDs using {request_url}")
         );
 
         let start_id = STORY_LIMIT * page;
@@ -227,8 +256,7 @@ impl HNClient {
             "{}/search?tags=story,({}){}&hitsPerPage={}",
             HN_ALGOLIA_PREFIX,
             ids.iter().fold("".to_owned(), |tags, story_id| format!(
-                "{}story_{},",
-                tags, story_id
+                "{tags}story_{story_id},"
             )),
             numeric_filters.query(),
             STORY_LIMIT,
@@ -239,30 +267,35 @@ impl HNClient {
                 .get(&request_url)
                 .call()?
                 .into_json::<StoriesResponse>()?,
-            format!(
-                "get stories (tag=front_page, by_date=false, page={}) using {}",
-                page, request_url
-            )
+            format!("get stories (tag={tag}, page={page}) using {request_url}",)
         );
 
-        Ok(self.reoder_front_page_stories(response.into(), ids))
+        Ok(self.reorder_stories_based_on_ids(response.into(), ids))
     }
 
-    /// Get a list of stories filtering on a specific tag
+    /// Get a list of stories filtering on a specific tag.
+    ///
+    /// Depending on the specifed `sort_mode`, stories are retrieved based on
+    /// the Algolia API or a combination of Algolia API and the Official API.
     pub fn get_stories_by_tag(
         &self,
         tag: &str,
-        by_date: bool,
+        sort_mode: StorySortMode,
         page: usize,
         numeric_filters: query::StoryNumericFilters,
     ) -> Result<Vec<Story>> {
-        if tag == "front_page" {
-            return self.get_front_page_stories(page, numeric_filters);
-        }
+        let search_op = match sort_mode {
+            StorySortMode::None => {
+                return self.get_stories_no_sort(tag, page, numeric_filters);
+            }
+            StorySortMode::Date => "search_by_date",
+            StorySortMode::Points => "search", // Algolia API default search is sorted by points
+        };
+
         let request_url = format!(
             "{}/{}?tags={}&hitsPerPage={}&page={}{}",
             HN_ALGOLIA_PREFIX,
-            if by_date { "search_by_date" } else { "search" },
+            search_op,
             tag,
             STORY_LIMIT,
             page,
@@ -275,9 +308,9 @@ impl HNClient {
                 .call()?
                 .into_json::<StoriesResponse>()?,
             format!(
-                "get stories (tag={}, by_date={}, page={}, numeric_filters={}) using {}",
+                "get stories (tag={}, sort_mode={:?}, page={}, numeric_filters={}) using {}",
                 tag,
-                by_date,
+                sort_mode,
                 page,
                 numeric_filters.query(),
                 request_url
@@ -287,7 +320,6 @@ impl HNClient {
         Ok(response.into())
     }
 
-    /// gets a web article from a URL
     pub fn get_article(url: &str) -> Result<Article> {
         let article_parse_command = &config::get_config().article_parse_command;
         let output = std::process::Command::new(&article_parse_command.command)
@@ -316,6 +348,114 @@ impl HNClient {
             Err(anyhow::anyhow!(stderr))
         }
     }
+
+    pub fn login(&self, username: &str, password: &str) -> Result<()> {
+        info!("Trying to login, user={username}...");
+
+        let res = self
+            .client
+            .post(&format!("{HN_HOST_URL}/login"))
+            .set("mode", "no-cors")
+            .set("credentials", "include")
+            .set("Access-Control-Allow-Origin", "*")
+            .send_form(&[("acct", username), ("pw", password)])?
+            .into_string()?;
+
+        // determine that a login is successful by finding the logout button
+        if res.contains("href=\"logout") {
+            Ok(())
+        } else {
+            Err(anyhow::anyhow!("Bad login"))
+        }
+    }
+
+    pub fn get_story_page_content(&self, story_id: u32) -> Result<String> {
+        let morelink_rg = regex::Regex::new("<a.*?href='(?P<link>.*?)'.*class='morelink'.*?>")?;
+
+        let mut content = self
+            .client
+            .get(&format!("{HN_HOST_URL}/item?id={story_id}"))
+            .call()?
+            .into_string()?;
+
+        // The story returned by HN can have multiple pages,
+        // we need to make additional requests for each page and
+        // concatenate all the responses to get the story's whole content.
+        let mut curr_page_content = content.clone();
+
+        while let Some(cap) = morelink_rg.captures(&curr_page_content) {
+            let next_page_link = cap.name("link").unwrap().as_str().replace("&amp;", "&");
+
+            let next_page_content = self
+                .client
+                .get(&format!("{HN_HOST_URL}/{next_page_link}"))
+                .call()?
+                .into_string()?;
+
+            content.push_str(&next_page_content);
+            curr_page_content = next_page_content;
+        }
+
+        Ok(content)
+    }
+
+    /// Parse a story's vote data
+    ///
+    /// The vote data is represented by a hashmap from `id` to a struct consisting of
+    /// `auth` and `upvoted` (false=no vote, true=has vote), in which `id` is
+    /// is an item's id and `auth` is a string for authentication purpose when voting.
+    pub fn parse_story_vote_data(&self, page_content: &str) -> Result<HashMap<String, VoteData>> {
+        let upvote_rg =
+            regex::Regex::new("<a.*?id='up_(?P<id>.*?)'.*?auth=(?P<auth>[0-9a-z]*).*?>")?;
+        let unvote_rg =
+            regex::Regex::new("<a.*?id='un_(?P<id>.*?)'.*?auth=(?P<auth>[0-9a-z]*).*?>")?;
+
+        let mut hm = HashMap::new();
+
+        upvote_rg.captures_iter(page_content).for_each(|c| {
+            let id = c.name("id").unwrap().as_str().to_owned();
+            let auth = c.name("auth").unwrap().as_str().to_owned();
+            hm.insert(
+                id,
+                VoteData {
+                    auth,
+                    upvoted: false,
+                },
+            );
+        });
+
+        unvote_rg.captures_iter(page_content).for_each(|c| {
+            let id = c.name("id").unwrap().as_str().to_owned();
+            let auth = c.name("auth").unwrap().as_str().to_owned();
+            hm.insert(
+                id,
+                VoteData {
+                    auth,
+                    upvoted: true,
+                },
+            );
+        });
+
+        Ok(hm)
+    }
+
+    /// Vote a HN item.
+    ///
+    /// Depending on the vote status (`upvoted`), the function will make
+    /// either an "upvote" or "unvote" request.
+    pub fn vote(&self, id: u32, auth: &str, upvoted: bool) -> Result<()> {
+        log!(
+            {
+                let vote_url = format!(
+                    "{HN_HOST_URL}/vote?id={id}&how={}&auth={auth}",
+                    if !upvoted { "up" } else { "un" }
+                );
+                self.client.get(&vote_url).call()?;
+            },
+            format!("vote HN item (id={id})")
+        );
+        Ok(())
+    }
 }
 
 pub fn init_client() -> &'static HNClient {
