@@ -1,20 +1,30 @@
 // modules
 pub mod client;
 pub mod config;
+pub mod model;
+pub mod parser;
 pub mod prelude;
 pub mod utils;
 pub mod view;
 
 const DEFAULT_CONFIG_FILE: &str = "hn-tui.toml";
+const DEFAULT_AUTH_FILE: &str = "hn-auth.toml";
 const DEFAULT_LOG_FILE: &str = "hn-tui.log";
 
 use clap::*;
 use prelude::*;
 
-fn run() {
+fn run(auth: Option<config::Auth>) {
     // setup HN Client
     let client = client::init_client();
 
+    // login if authentication is specified
+    if let Some(auth) = auth {
+        if let Err(err) = client.login(&auth.username, &auth.password) {
+            tracing::warn!("Failed to login, user={}: {err}", auth.username);
+        }
+    }
+
     // setup the application's UI
     let s = view::init_ui(client);
 
@@ -42,7 +52,7 @@ fn init_logging(log_dir_str: &str) {
     }
 
     let log_file = std::fs::File::create(log_dir.join(DEFAULT_LOG_FILE)).unwrap_or_else(|err| {
-        panic!("failed to create application's log file: {}", err);
+        panic!("failed to create application's log file: {err}");
     });
 
     tracing_subscriber::fmt::fmt()
@@ -53,22 +63,26 @@ fn init_logging(log_dir_str: &str) {
 }
 
 /// parse command line arguments
-fn parse_args(config_dir: &std::path::Path, cache_dir: &std::path::Path) -> ArgMatches {
+fn parse_args(config_dir: std::path::PathBuf, cache_dir: std::path::PathBuf) -> ArgMatches {
     Command::new("hackernews-tui")
-        .version("0.10.2")
+        .version("0.12.0")
         .author("Thang Pham <phamducthang1234@gmail>")
         .about("A Terminal UI to browse Hacker News")
+        .arg(
+            Arg::new("auth")
+                .short('a')
+                .long("auth")
+                .value_name("FILE")
+                .default_value(config_dir.join(DEFAULT_AUTH_FILE).into_os_string())
+                .help("Path to the application's authentication file")
+                .next_line_help(true),
+        )
         .arg(
             Arg::new("config")
                 .short('c')
                 .long("config")
                 .value_name("FILE")
-                .default_value(
-                    config_dir
-                        .join(DEFAULT_CONFIG_FILE)
-                        .to_str()
-                        .expect("failed to config file `Path` to str"),
-                )
+                .default_value(config_dir.join(DEFAULT_CONFIG_FILE).into_os_string())
                 .help("Path to the application's config file")
                 .next_line_help(true),
         )
@@ -77,11 +91,7 @@ fn parse_args(config_dir: &std::path::Path, cache_dir: &std::path::Path) -> ArgM
                 .short('l')
                 .long("log")
                 .value_name("FOLDER")
-                .default_value(
-                    cache_dir
-                        .to_str()
-                        .expect("failed to convert cache dir `Path` to str"),
-                )
+                .default_value(cache_dir.into_os_string())
                 .help("Path to a folder to store application's logs")
                 .next_line_help(true),
         )
@@ -102,17 +112,33 @@ fn init_app_dirs() -> (std::path::PathBuf, std::path::PathBuf) {
     (config_dir, cache_dir)
 }
 
+fn init_auth(auth_file_str: &str) -> Option<config::Auth> {
+    match config::Auth::from_file(auth_file_str) {
+        Ok(auth) => Some(auth),
+        Err(err) => {
+            tracing::warn!("Failed to get authentication from {}: {err}", auth_file_str);
+            None
+        }
+    }
+}
+
 fn main() {
     let (config_dir, cache_dir) = init_app_dirs();
-    let args = parse_args(&config_dir, &cache_dir);
+    let args = parse_args(config_dir, cache_dir);
 
     init_logging(
-        args.value_of("log")
+        args.get_one::<String>("log")
             .expect("`log` argument should have a default value"),
     );
+
     config::load_config(
-        args.value_of("config")
+        args.get_one::<String>("config")
             .expect("`config` argument should have a default value"),
     );
-    run();
+
+    let auth = init_auth(
+        args.get_one::<String>("auth")
+            .expect("`auth` argument should have a default value"),
+    );
+    run(auth);
 }
