@@ -7,16 +7,16 @@ use cursive_async_view::AsyncView;
 pub fn construct_comment_view_async(
     siv: &mut Cursive,
     client: &'static client::HNClient,
-    story: &client::Story,
+    story: &Story,
 ) -> impl View {
     let id = story.id;
 
-    AsyncView::new_with_bg_creator(siv, move || Ok(client.lazy_load_story_comments(id)), {
+    AsyncView::new_with_bg_creator(siv, move || Ok(client.get_story_hidden_data(id)), {
         let story = story.clone();
         move |result: Result<_>| {
             ResultView::new(
-                result.with_context(|| format!("failed to load comments from story (id={})", id)),
-                |receiver| comment_view::construct_comment_view(&story, receiver),
+                result.with_context(|| format!("failed to load comments from story (id={id})")),
+                |receiver| comment_view::construct_comment_view(client, &story, receiver),
             )
         }
     })
@@ -29,23 +29,22 @@ pub fn construct_story_view_async(
     siv: &mut Cursive,
     client: &'static client::HNClient,
     tag: &'static str,
-    by_date: bool,
+    sort_mode: client::StorySortMode,
     page: usize,
     numeric_filters: client::StoryNumericFilters,
 ) -> impl View {
     AsyncView::new_with_bg_creator(
         siv,
-        move || Ok(client.get_stories_by_tag(tag, by_date, page, numeric_filters)),
+        move || Ok(client.get_stories_by_tag(tag, sort_mode, page, numeric_filters)),
         move |result| {
             ResultView::new(
                 result.with_context(|| {
                     format!(
-                        "failed to get stories (tag={}, by_date={}, page={}, numeric_filters={{{}}})",
-                        tag, by_date, page, numeric_filters,
+                        "failed to get stories (tag={tag}, sort_mode={sort_mode:?}, page={page}, numeric_filters={{{numeric_filters}}})",
                     )
                 }),
                 |stories| {
-                    story_view::construct_story_view(stories, client, tag, by_date, page, numeric_filters)
+                    story_view::construct_story_view(stories, client, tag, sort_mode, page, numeric_filters)
                 },
             )
         },
@@ -90,7 +89,7 @@ fn animation(width: usize, _height: usize, frame_idx: usize) -> cursive_async_vi
         let factor = (frame_idx as f64) / (n_frames as f64);
         let x = (factor * width as f64) as usize;
 
-        let content = crate::utils::combine_styled_strings(vec![
+        let content = crate::utils::combine_styled_strings([
             StyledString::styled(repeat_str("- ", x / 2), style),
             StyledString::styled('ᗧ', style),
             StyledString::styled(repeat_str(" o", width.saturating_sub(x + 1) / 2), style),
@@ -105,7 +104,7 @@ fn animation(width: usize, _height: usize, frame_idx: usize) -> cursive_async_vi
         let factor = (frame_idx as f64) / (n_frames as f64);
         let x = (factor * width as f64) as usize;
 
-        let content = crate::utils::combine_styled_strings(vec![
+        let content = crate::utils::combine_styled_strings([
             StyledString::styled(repeat_str(symbol, x), style.back),
             StyledString::styled(repeat_str(symbol, width - x), style.front),
         ]);
