@@ -9,7 +9,7 @@ static STORY_TAGS: [&str; 5] = ["front_page", "story", "ask_hn", "show_hn", "job
 /// StoryView is a View displaying a list stories corresponding
 /// to a particular category (top stories, newest stories, most popular stories, etc).
 pub struct StoryView {
-    pub stories: Vec<client::Story>,
+    pub stories: Vec<Story>,
 
     view: ScrollView<LinearLayout>,
     raw_command: String,
@@ -20,7 +20,7 @@ impl ViewWrapper for StoryView {
 }
 
 impl StoryView {
-    pub fn new(stories: Vec<client::Story>, starting_id: usize) -> Self {
+    pub fn new(stories: Vec<Story>, starting_id: usize) -> Self {
         StoryView {
             view: Self::construct_story_view(&stories, starting_id),
             stories,
@@ -28,11 +28,8 @@ impl StoryView {
         }
     }
 
-    fn construct_story_view(
-        stories: &[client::Story],
-        starting_id: usize,
-    ) -> ScrollView<LinearLayout> {
-        // Determine the maximum length of a story's ID string.
+    fn construct_story_view(stories: &[Story], starting_id: usize) -> ScrollView<LinearLayout> {
+        // Determine the maximum length of a story's ID.
         // This maximum length is used to align the display of the story IDs.
         let max_id_len = {
             let max_id = starting_id + stories.len() + 1;
@@ -49,6 +46,7 @@ impl StoryView {
         LinearLayout::vertical()
             .with(|s| {
                 stories.iter().enumerate().for_each(|(i, story)| {
+                    // initialize the story text with its ID
                     let mut story_text = StyledString::styled(
                         format!("{1:>0$}. ", max_id_len, starting_id + i + 1),
                         config::get_config_theme().component_style.metadata,
@@ -62,13 +60,13 @@ impl StoryView {
     }
 
     /// Get the text summarizing basic information about a story
-    fn get_story_text(max_id_len: usize, story: &client::Story) -> StyledString {
-        let mut story_text = story.title.clone();
+    fn get_story_text(max_id_len: usize, story: &Story) -> StyledString {
+        let mut story_text = story.styled_title();
 
         if let Ok(url) = url::Url::parse(&story.url) {
             if let Some(domain) = url.domain() {
                 story_text.append_styled(
-                    format!(" ({})", domain),
+                    format!(" ({domain})"),
                     config::get_config_theme().component_style.link,
                 );
             }
@@ -77,8 +75,8 @@ impl StoryView {
         story_text.append_plain("\n");
 
         story_text.append_styled(
-            // left-align the story's metadata by `max_id_len+2`
-            // which is the width of the string "{max_story_id}. "
+            // left-align the story's metadata by `max_id_len+2`,
+            // which is the maximum width of a string `{story_id}. `
             format!(
                 "{:width$}{} points | by {} | {} ago | {} comments",
                 " ",
@@ -126,7 +124,7 @@ impl ScrollViewContainer for StoryView {
 }
 
 pub fn construct_story_main_view(
-    stories: Vec<client::Story>,
+    stories: Vec<Story>,
     client: &'static client::HNClient,
     starting_id: usize,
 ) -> OnEventView<StoryView> {
@@ -221,7 +219,7 @@ pub fn construct_story_main_view(
         .on_scroll_events()
 }
 
-fn get_story_view_title_bar(tag: &'static str) -> impl View {
+fn get_story_view_title_bar(tag: &'static str, sort_mode: client::StorySortMode) -> impl View {
     let style = config::get_config_theme().component_style.title_bar;
     let mut title = StyledString::styled(
         "[Y]",
@@ -234,8 +232,13 @@ fn get_story_view_title_bar(tag: &'static str) -> impl View {
     for (i, item) in STORY_TAGS.iter().enumerate() {
         title.append_styled(" | ", style);
         if *item == tag {
+            let sort_mode_desc = match sort_mode {
+                client::StorySortMode::None => "",
+                client::StorySortMode::Date => " (by_date)",
+                client::StorySortMode::Points => " (by_point)",
+            };
             title.append_styled(
-                format!("{}.{}", i + 1, item),
+                format!("{}.{}{}", i + 1, item, sort_mode_desc),
                 Style::from(style)
                     .combine(config::get_config_theme().component_style.current_story_tag),
             );
@@ -256,10 +259,10 @@ fn get_story_view_title_bar(tag: &'static str) -> impl View {
 
 /// Construct a story view given a list of stories.
 pub fn construct_story_view(
-    stories: Vec<client::Story>,
+    stories: Vec<Story>,
     client: &'static client::HNClient,
     tag: &'static str,
-    by_date: bool,
+    sort_mode: client::StorySortMode,
     page: usize,
     numeric_filters: client::StoryNumericFilters,
 ) -> impl View {
@@ -267,7 +270,7 @@ pub fn construct_story_view(
     let main_view = construct_story_main_view(stories, client, starting_id).full_height();
 
     let mut view = LinearLayout::vertical()
-        .child(get_story_view_title_bar(tag))
+        .child(get_story_view_title_bar(tag, sort_mode))
         .child(main_view)
         .child(utils::construct_footer_view::<StoryView>());
     view.set_focus_index(1)
@@ -276,7 +279,7 @@ pub fn construct_story_view(
     let current_tag_pos = STORY_TAGS
         .iter()
         .position(|t| *t == tag)
-        .unwrap_or_else(|| panic!("unkwnown tag {}", tag));
+        .unwrap_or_else(|| panic!("unkwnown tag {tag}"));
 
     let story_view_keymap = config::get_story_view_keymap().clone();
 
@@ -288,31 +291,49 @@ pub fn construct_story_view(
         .on_pre_event(config::get_global_keymap().open_help_dialog.clone(), |s| {
             s.add_layer(StoryView::construct_on_event_help_view())
         })
-        .on_pre_event(story_view_keymap.toggle_sort_by_date, move |s| {
+        .on_pre_event(story_view_keymap.cycle_sort_mode, move |s| {
             // disable "search_by_date" for front_page stories
             if tag == "front_page" {
                 return;
             }
-            construct_and_add_new_story_view(s, client, tag, !by_date, 0, numeric_filters, true);
-        })
-        // story tag navigation
-        .on_pre_event(story_view_keymap.next_story_tag, move |s| {
             construct_and_add_new_story_view(
                 s,
                 client,
-                STORY_TAGS[(current_tag_pos + 1) % STORY_TAGS.len()],
+                tag,
+                sort_mode.next(tag),
+                0,
+                numeric_filters,
                 true,
+            );
+        })
+        // story tag navigation
+        .on_pre_event(story_view_keymap.next_story_tag, move |s| {
+            let next_tag = STORY_TAGS[(current_tag_pos + 1) % STORY_TAGS.len()];
+            construct_and_add_new_story_view(
+                s,
+                client,
+                next_tag,
+                if next_tag == "story" || next_tag == "job" {
+                    client::StorySortMode::Date
+                } else {
+                    client::StorySortMode::None
+                },
                 0,
                 StoryNumericFilters::default(),
                 false,
             );
         })
         .on_pre_event(story_view_keymap.prev_story_tag, move |s| {
+            let prev_tag = STORY_TAGS[(current_tag_pos + STORY_TAGS.len() - 1) % STORY_TAGS.len()];
             construct_and_add_new_story_view(
                 s,
                 client,
-                STORY_TAGS[(current_tag_pos + STORY_TAGS.len() - 1) % STORY_TAGS.len()],
-                true,
+                prev_tag,
+                if prev_tag == "story" || prev_tag == "job" {
+                    client::StorySortMode::Date
+                } else {
+                    client::StorySortMode::None
+                },
                 0,
                 StoryNumericFilters::default(),
                 false,
@@ -325,7 +346,7 @@ pub fn construct_story_view(
                     s,
                     client,
                     tag,
-                    by_date,
+                    sort_mode,
                     page - 1,
                     numeric_filters,
                     true,
@@ -337,7 +358,7 @@ pub fn construct_story_view(
                 s,
                 client,
                 tag,
-                by_date,
+                sort_mode,
                 page + 1,
                 numeric_filters,
                 true,
@@ -350,13 +371,13 @@ pub fn construct_and_add_new_story_view(
     s: &mut Cursive,
     client: &'static client::HNClient,
     tag: &'static str,
-    by_date: bool,
+    sort_mode: client::StorySortMode,
     page: usize,
     numeric_filters: client::StoryNumericFilters,
     pop_layer: bool,
 ) {
     let async_view =
-        async_view::construct_story_view_async(s, client, tag, by_date, page, numeric_filters);
+        async_view::construct_story_view_async(s, client, tag, sort_mode, page, numeric_filters);
     if pop_layer {
         s.pop_layer();
     }
